"""
    This module is for testing the code_executor module.
"""

__author__ = 'coty'

import unittest
import os
from etltest.code_executor import CodeExecutor


class code_executor_tests(unittest.TestCase):

    def setUp(self):
        self.one_dir_path = self.setup_oneFile()
        self.multi_dir_path = self.setup_twoFiles()

    def setup_oneFile(self):
        #Create a single file for testing purposes.
        from tempfile import mkdtemp, mkstemp
        dir_path = mkdtemp()

        (fh, f_path) = mkstemp(suffix='.py', dir=dir_path)
        f = os.fdopen(fh, 'w')
        f.write("#!/usr/bin/python\n\nimport sys\nsys.exit(0)")
        os.chmod(f_path, 0770)
        f.close()

        return dir_path

    def setup_twoFiles(self):
        #Create two temporary files for testing purposes.
        from tempfile import mkdtemp, mkstemp
        dir_path = mkdtemp()

        (fh, f_path) = mkstemp(suffix='.py', dir=dir_path)
        f = os.fdopen(fh, 'w')
        f.write("#!/usr/bin/python\n\nimport sys\nsys.exit(0)")
        os.chmod(f_path, 0770)
        f.close()

        (fh, f_path) = mkstemp(suffix='.py', dir=dir_path)
        f = os.fdopen(fh, 'w')
        f.write("#!/usr/bin/python\n\nimport sys\nsys.exit(0)")
        os.chmod(f_path, 0770)
        f.close()

        return dir_path

    def tearDown(self):
        #Remove all testing files.
        from shutil import rmtree
        rmtree(self.one_dir_path)
        rmtree(self.multi_dir_path)

    def test_init(self):
        ce = CodeExecutor("test")
        self.assertEqual("test", ce.out_dir)

    def test_execute_one_test_mode(self):
        ce = CodeExecutor(self.one_dir_path)
        self.assertEqual(ce.execute(True), 0)

    def test_execute_multi_test_mode(self):
        ce = CodeExecutor(self.multi_dir_path)
        self.assertEqual(ce.execute(True), 0)

    def test_execute_one(self):
        ce = CodeExecutor(self.one_dir_path)
        self.assertEqual(ce.execute(False), None)

    def test_execute_multi(self):
        ce = CodeExecutor(self.multi_dir_path)
        self.assertEqual(ce.execute(False), None)





import sys
import subprocess


class executeTest():

    def execute(self):
        #p = subprocess.Popen(['echo', 'Hello!'], stdout=subprocess.PIPE)
        p = subprocess.Popen(['python', '/home/ameadows/Documents/etlTest/output/DataMart/UsersDim.py'], stdout=subprocess.PIPE)

        print p.communicate()

if __name__ == "__main__":
    executeTest().execute()


__author__ = 'ameadows'
"""
    This file contains all the logic necessary to execute ETL code based on the settings stored in the tools settings
    file.  Process execution should occur after the test data is loaded and before the unit tests are run.
"""
import logging
import os
import paramiko
from etltest.utilities.settings_manager import SettingsManager

class ProcessExecutor():

    def __init__(self, tool_name):

        """
            Here we initialize the process executor object with the necessary customized logging.
            :param: tool_name The tool that is being used for running the data integration process.  This is the name of the
            tool in the tool configuration file (default is tools.yml)
            :param: tool_name str
        """
        from etltest.utilities.settings import etltest_config, console
        self.log = logging.getLogger(name="ProcessExecutor")
        self.log.setLevel(etltest_config['logging_level'])
        self.log.addHandler(console)

        self.tool_name = tool_name
        self.tool = SettingsManager().get_tool(tool_name)
        self.log.info(u"Setting up tool {0:s}: {1:s}".format(tool_name, self.tool))

        self.local_names = ['localhost', '127.0.0.1']

    def create_secure_shell(self):
        """
        Create a secured shell connection to the data integration server, based on the connection info provided from
        the tool configuration file.
        :return: ssh connection.
        """
        ssh = paramiko.SSHClient()

        if ('private_key' in self.tool and self.tool['private_key'] is None) or 'private_key' not in self.tool:
            ssh.connect(self.tool['host_name'], port=self.tool['port'],
                        username=self.tool['user_name'], password=self.tool['password'])
        elif self.tool['host_name'] is not None:
            pkey = paramiko.RSAKey.from_private_key_file(self.tool['private_key'])
            ssh.set_missing_host_key_policy(paramiko.WarningPolicy())

            if self.tool['port'] is not None:
                ssh.connect(self.tool['host_name'], port=self.tool['port'], username=self.tool['user_name'],
                            password=self.tool['password'], pkey=pkey)
            else:
                ssh.connect(self.tool['host_name'], username=self.tool['user_name'],
                            password=self.tool['password'], pkey=pkey)
        else:
            raise Exception("The connection information is incorrect or not allowed.")

        return ssh

    def read_output(self, stdout, stderr):
        """
        Reads the return values of the executed process and writes them to the log.
        :param stdout:
        :param stderr:
        :return:
        """

        for line in stderr:
            self.log.error(line)

        for line in stdout:
            self.log.info(line)


    def execute_process(self, process_type, process_name):

        self.log.info(u"Attempting to run {0:s} with {1:s}".format(process_name, self.tool_name))

        from subprocess import call

        tool_path = SettingsManager().system_variable_replace(self.tool['tool_path'])

        process_param = self.tool['process_param']
        params = self.tool['params']
        self.log.info(u"Using {0:s} with {1:s}".format(process_param, params))

        process = process_param + process_name
        self.log.info(u"Running {0:s}".format(process))

        for type in self.tool['script_types']:
            if type['type'] == process_type:
                tool_script = type['script']

        self.log.info(u"Using {0:s} with {1:s}".format(tool_path, tool_script))

        tool_path_script = os.path.join(tool_path, tool_script)

        if self.tool['host_name'] not in self.local_names:
            ssh = self.create_secure_shell()

            self.log.debug("Attempting to change directory to {0:s}".format(tool_path))
            stdin, stdout, stderr = ssh.exec_command("cd " + tool_path)

            self.read_output(stdout, stderr)

            stdin, stdout, stderr = ssh.exec_command(tool_script + " " + process)

            self.read_output(stdout, stderr)
            ssh.close()
        else:

            return call([tool_path_script, process])




__author__ = 'coty, ameadows'

import logging
import os

from utilities.settings import etltest_config, console
from utilities.settings_manager import SettingsManager
from utilities.yaml_parser import YAMLParser


class CodeGenerator():

    def __init__(self, in_file=None, in_dir=None, out_dir=None, test_run=None):
        """
        Generates full test code sets based on yaml files and the
          Jinja2 templates.  in_file and in_dir are mutually exclusive.
        :param in_file:  The single file to be processed.  Can not be set if in_dir is set.
        :param in_file:  str
        :param in_dir:  The directory to be processed.  Can not be set if in_file is set.
        :param in_dir:  str
        """
        self.log = logging.getLogger(name="CodeGenerator")
        self.log.setLevel(etltest_config['logging_level'])
        self.log.addHandler(console)

        self.in_file = in_file
        self.in_dir = in_dir
        self.out_dir = out_dir

        self.test_dir = SettingsManager().find_setting('Locations', 'tests')
        self.data_dir = SettingsManager().find_setting('Locations', 'data')


        if self.in_file is not None:
            self.test_list = YAMLParser().read_file(self.in_file)
        elif self.in_dir is not None:
            self.test_list = YAMLParser().read_dir(self.in_dir)
        else:
            test_loc = SettingsManager().find_setting("Locations", "tests")
            self.test_list = YAMLParser().read_dir(test_loc)

        if self.out_dir is None:
            self.out_dir = SettingsManager().find_setting('Locations', 'output')

    def generate_test(self):
        """
            This method generates test code based on the Jinja2 template and the test yaml file provided.
        """
        # TODO:  Continue working on YAML Validator.  Must be included either here or in the parser...

        self.jinja_setup()
        self.template = self.jinja_env.get_template("output/test.jinja2")

        for group in self.test_list:
            self.test_group, self.tests = group.popitem()
            self.file_path, self.filename = self.test_group.rsplit("\\", 1)      # Using the testGroup as the
            self.filename += ".py"                                               # folder structure for output.
            self.file_path = os.path.join(self.out_dir, self.file_path)
            self.test_group = self.test_group.replace("\\", '')  # Removing slashes so the test class is properly
                                                                 # named.

            self.log.debug("Test Group Name: %s" % self.test_group)
            self.log.debug("Tests for test group: %s" % self.tests)
            self.log.debug("File path: %s" % self.file_path)
            self.log.debug("File name: %s" % self.filename)

            self.variables = {
                "header": self.header,
                "tests": self.tests,
                "testGroup": self.test_group
                         }

            if not os.path.isdir(self.file_path):
                os.makedirs(self.file_path, 0755)
                self.log.debug("%s directory created." % self.file_path)

            os.chdir(self.file_path)
            with open(self.filename, 'w') as f:
                f.write(self.template.render(self.variables))
                f.close()

            self.log.info(u"{0:s} test file generated.".format(self.filename))


    def jinja_setup(self):
        from jinja2 import Environment, FileSystemLoader
        from time import strftime, gmtime
        template_dir = os.path.join(SettingsManager().get_file_location(), 'etltest/templates')

        self.log.debug(u"Attempting to process templates from {0:s}".format(template_dir))

        self.jinja_env = Environment(loader=FileSystemLoader(template_dir),
                                     trim_blocks=True, lstrip_blocks=True)

        # Header lines created here and added to the templates as required
        self.header = "#!/usr/bin/python\n" \
                 "#\n" \
                 "# This file was created by etlTest.\n" \
                 "#\n"


# __author__ = 'ameadows'
#
# import unittest
# import os
# import subprocess
# import sys
#
# from etltest.utilities.settings_manager import SettingsManager
# from etltest import etlTest
#
#
# class CommandLineTestCase(unittest.TestCase):
#
#     @classmethod
#     def setUpClass(cls):
#         parser = etlTest.create_parser()
#         cls.parser = parser
#
#
# class etlTestTests(CommandLineTestCase):
#
#     def setUp(self):
#         SettingsManager().first_run_test()
#
#         self.in_file = SettingsManager().system_variable_replace("${"
#                                                                  "ETL_TEST_ROOT}/etltest/samples/test/dataMart/users_dim.yml")
#         self.in_dir = SettingsManager().system_variable_replace("${ETL_TEST_ROOT}/etltest/samples/test/dataMart/")
#
#         self.expected_output = SettingsManager().system_variable_replace("${"
#                                                                          "ETL_TEST_ROOT}/etltest/samples/output/DataMart/UsersDim.py")
#         self.process = os.path.join(SettingsManager().get_file_location(), "etltest/etlTest.py")
#
#     def test_in_file_generation_output(self):
#         file_param = "-f {0:s}".format(self.in_file)
#         output_file = os.path.join(SettingsManager().get_file_location(),
#                                        'etltest/samples/output/main/in_file_generation.txt')
#
#         given_result = subprocess.check_output(args=['python', self.process, file_param, "-g"])
#
#         with open(output_file, 'r') as f:
#             expected_result = f.read()
#
#         self.assertEqual(given_result, expected_result)
#
#     def test_in_dir_generation_output(self):
#         file_param = "-d {0:s}".format(self.in_dir)
#         output_file = os.path.join(SettingsManager().get_file_location(),
#                                    'etltest/samples/output/main/in_dir_generation.txt')
#
#         given_result = subprocess.check_output(args=['python', self.process, file_param, "-g"])
#
#         with open(output_file, 'r') as f:
#             expected_result = f.read()
#
#         self.assertEqual(given_result, expected_result)
#
#     def test_in_file_custom_output_generation_output(self):
#         file_param = "-f {0:s}".format(self.in_file)
#         output_file = os.path.join(SettingsManager().get_file_location(),
#                                        'etltest/samples/output/main/in_file_generation.txt')
#
#         given_result = subprocess.check_output(args=['python', self.process, file_param, "-g"])
#
#         with open(output_file, 'r') as f:
#             expected_result = f.read()
#
#         self.assertEqual(given_result, expected_result)
#
#     def test_parser_no_options(self):
#         # Test the results if there are no options given.
#         # Should return the help listing.
#         given_result = subprocess.check_output(args=['python', self.process])
#         output_file = os.path.join(SettingsManager().get_file_location(), 'etltest/samples/output/main/no_args.txt')
#
#         with open(output_file, 'r') as f:
#             expected_result = f.read()
#
#         self.assertEqual(given_result, expected_result)
#
#     def test_parser_in_file_in_dir_exclusivity(self):
#         #Test if in_file and in_dir are mutually exclusive.
#         file_param = "-f {0:s}".format(self.in_file)
#         dir_param = "-d {0:s}".format(self.in_dir)
#
#         with self.assertRaises(subprocess.CalledProcessError):
#             subprocess.check_output(args=['python', self.process, file_param, dir_param, "-g"])
#
#     def test_parser_no_args(self):
#         #Test if no args returns help options.
#         subprocess.check_output(args=['python', self.process], stderr=subprocess.STDOUT)
#
#     def test_parser_in_file_generation(self):
#         #Test if in_file generation exits appropriately.
#         subprocess.check_output(args=['python', self.process, '-f', self.in_file, '-g'])
#
#     def test_parser_in_dir_generation(self):
#         #Test if in_dir generation exits appropriately.
#         subprocess.check_output(args=['python', self.process, '-d', self.in_dir, '-g'])
#
#     def test_parser_in_file_custom_output_generation(self):
#         #Test if custom output generation exits appropriately.
#         output_loc = SettingsManager().find_setting('Locations', 'output')
#         subprocess.check_output(args=['python', self.process, '-f', self.in_file, '-g', '-o', str(output_loc)])
#
#     def test_parser_test_execution(self):
#         subprocess.check_output(args=['python', self.process, '-e'])



"""
    This module tests the yaml_parser module.
"""

__author__ = 'ameadows'

import unittest
import datetime
import os

from etltest.utilities.yaml_parser import YAMLParser
from etltest.utilities.settings_manager import SettingsManager


class YamlParserTests(unittest.TestCase):

    def setUp(self):
        SettingsManager().first_run_test()
        self.tmp_path = SettingsManager().get_file_location()
        self.data_loc = SettingsManager().find_setting('Locations', 'data')
        self.test_dir = self.tmp_path + "/etltest/samples/test/"
        self.data_dir = self.tmp_path + "/etltest/samples/data/"
        self.test_file = self.test_dir + "dataMart/users_dim.yml"
        self.data_file = self.data_dir + "etlUnitTest/users.yml"
        self.maxDiff = None

    def test_process_test_file_whitespace(self):
        self.test_file = " " + self.test_file
        given_result = YAMLParser().read_file(self.test_file)
        expected_result = [{'DataMart\\UsersDim': {'suites': [{'suite': 'dataMart'}], 'processes': [{'tool': 'PDI', 'processes': [{'type': 'job', 'name': 'data_mart/user_dim_jb.kjb'}]}], 'tests': [{'query': {'from': 'user_dim', 'where': 'user_id = 2', 'result': {'first_name': 'sarah'}, 'select': 'first_name', 'source': 'etlUnitTest'}, 'type': 'NotEqual', 'name': 'testFirstNameNotLower', 'desc': 'Ensures that the first name field is not lower case after being processed.'}, {'query': {'from': 'user_dim', 'where': 'user_id = 2', 'result': {'first_name': 'SARAH'}, 'select': 'first_name', 'source': 'etlUnitTest'}, 'type': 'Equal', 'name': 'testFirstNameUpper', 'desc': 'Test for process that upper cases the first name field of a users table record.'}, {'query': {'from': 'user_dim', 'where': 'user_id IN (1, 2)', 'result': "{'birthday': datetime.date(2000, 1, 4)}, {'birthday': datetime.date(2000, 2, 2)}", 'select': 'birthday', 'source': 'etlUnitTest'}, 'name': 'testUserValidBirthday', 'desc': 'Test for valid birth dates.'}, {'query': {'from': 'users', 'where': 'user_id = 2', 'select': 'is_active', 'source': 'etlUnitTest'}, 'type': 'BooleanTrue', 'name': 'testIsActiveTrue', 'desc': 'Passes if is_active field is set to true.'}, {'query': {'from': 'users', 'where': 'user_id = 1', 'select': 'is_active', 'source': 'etlUnitTest'}, 'type': 'BooleanFalse', 'name': 'testIsActiveFalse', 'desc': 'Passes if is_active field is set to false.'}], 'dataset': [{'source': 'etlUnitTest', 'table': 'users', 'records': [1, 2]}]}}]

        self.assertItemsEqual(given_result, expected_result)

    # Testing test yaml file for correct processing.
    def test_process_test_file(self):
        given_result = YAMLParser().read_file(self.test_file)
        expected_result = [{'DataMart\\UsersDim': {'suites': [{'suite': 'dataMart'}], 'processes': [{'tool': 'PDI', 'processes': [{'type': 'job', 'name': 'data_mart/user_dim_jb.kjb'}]}], 'tests': [{'query': {'from': 'user_dim', 'where': 'user_id = 2', 'result': {'first_name': 'sarah'}, 'select': 'first_name', 'source': 'etlUnitTest'}, 'type': 'NotEqual', 'name': 'testFirstNameNotLower', 'desc': 'Ensures that the first name field is not lower case after being processed.'}, {'query': {'from': 'user_dim', 'where': 'user_id = 2', 'result': {'first_name': 'SARAH'}, 'select': 'first_name', 'source': 'etlUnitTest'}, 'type': 'Equal', 'name': 'testFirstNameUpper', 'desc': 'Test for process that upper cases the first name field of a users table record.'}, {'query': {'from': 'user_dim', 'where': 'user_id IN (1, 2)', 'result': "{'birthday': datetime.date(2000, 1, 4)}, {'birthday': datetime.date(2000, 2, 2)}", 'select': 'birthday', 'source': 'etlUnitTest'}, 'name': 'testUserValidBirthday', 'desc': 'Test for valid birth dates.'}, {'query': {'from': 'users', 'where': 'user_id = 2', 'select': 'is_active', 'source': 'etlUnitTest'}, 'type': 'BooleanTrue', 'name': 'testIsActiveTrue', 'desc': 'Passes if is_active field is set to true.'}, {'query': {'from': 'users', 'where': 'user_id = 1', 'select': 'is_active', 'source': 'etlUnitTest'}, 'type': 'BooleanFalse', 'name': 'testIsActiveFalse', 'desc': 'Passes if is_active field is set to false.'}], 'dataset': [{'source': 'etlUnitTest', 'table': 'users', 'records': [1, 2]}]}}]

        self.assertItemsEqual(given_result, expected_result)

    # Testing data yaml file for correct processing.
    def test_process_data_file(self):
        given_result = YAMLParser().read_file(self.data_file)
        expected_result = [{1: {'first_name': 'Bob', 'last_name': 'Richards', 'user_id': 1, 'zipcode': 55555, 'birthday': datetime.date(2000, 1, 4), 'is_active': 0}, 2: {'first_name': 'Sarah', 'last_name': 'Jenkins', 'user_id': 2, 'zipcode': 12345, 'birthday': datetime.date(2000, 2, 2), 'is_active': 1}, 3: {'first_name': 'Frank', 'last_name': 'Williams', 'user_id': 3, 'zipcode': 56789, 'birthday': datetime.date(1972, 3, 3), 'is_active': 0}, 4: {'first_name': None, 'last_name': 'Thomas', 'user_id': 4, 'zipcode': 44444, 'birthday': datetime.date(1923, 1, 4), 'is_active': 1}}]
        self.assertItemsEqual(given_result, expected_result)


    # Testing a directory of test yaml files to verify they are processed correctly.
    def test_process_test_dir(self):
        given_result = YAMLParser().read_dir(self.test_dir)
        expected_result = [{'DataMart\\UsersDim': {'processes': [{'processes': [{'name': 'data_mart/user_dim_jb.kjb', 'type': 'job'}], 'tool': 'PDI'}], 'dataset': [{'source': 'etlUnitTest', 'table': 'users', 'records': [1, 2]}], 'tests': [{'name': 'testFirstNameNotLower', 'query': {'source': 'etlUnitTest', 'where': 'user_id = 2', 'from': 'user_dim', 'result': {'first_name': 'sarah'}, 'select': 'first_name'}, 'type': 'NotEqual', 'desc': 'Ensures that the first name field is not lower case after being processed.'}, {'name': 'testFirstNameUpper', 'query': {'source': 'etlUnitTest', 'where': 'user_id = 2', 'from': 'user_dim', 'result': {'first_name': 'SARAH'}, 'select': 'first_name'}, 'type': 'Equal', 'desc': 'Test for process that upper cases the first name field of a users table record.'}, {'name': 'testUserValidBirthday', 'query': {'source': 'etlUnitTest', 'where': 'user_id IN (1, 2)', 'from': 'user_dim', 'result': "{'birthday': datetime.date(2000, 1, 4)}, {'birthday': datetime.date(2000, 2, 2)}", 'select': 'birthday'}, 'desc': 'Test for valid birth dates.'}, {'name': 'testIsActiveTrue', 'query': {'source': 'etlUnitTest', 'where': 'user_id = 2', 'from': 'users', 'select': 'is_active'}, 'type': 'BooleanTrue', 'desc': 'Passes if is_active field is set to true.'}, {'name': 'testIsActiveFalse', 'query': {'source': 'etlUnitTest', 'where': 'user_id = 1', 'from': 'users', 'select': 'is_active'}, 'type': 'BooleanFalse', 'desc': 'Passes if is_active field is set to false.'}], 'suites': [{'suite': 'dataMart'}]}}]

        self.assertEqual(given_result, expected_result)

    # Testing a directory of data yaml files to verify they are processed correctly.
    def test_process_data_dir(self):
        given_result = YAMLParser().read_dir(self.data_dir)
        expected_result = [{1: {'city': 'Young America', 'state': 'Minnesota', 'user_geo_ref_id': 1, 'zipcode': 55555, 'country': 'United States'}, 2: {'city': 'Schenectady', 'state': 'New York', 'user_geo_ref_id': 2, 'zipcode': 12345, 'country': 'United States'}, 3: {'city': 'Colton', 'state': 'California', 'user_geo_ref_id': 3, 'zipcode': 56789, 'country': 'United States'}, 4: {'city': 'Newton Falls', 'state': 'Ohio', 'user_geo_ref_id': 4, 'zipcode': 44444, 'country': 'United States'}},
                           {'Young America': {'city': 'Young America', 'state': 'Minnesota', 'user_geo_ref_id': 1, 'zipcode': 55555, 'country': 'United States'}, 'Colton': {'city': 'Colton', 'state': 'California', 'user_geo_ref_id': 3, 'zipcode': 56789, 'country': 'United States'}, 'Schenectady': {'city': 'Schenectady', 'state': 'New York', 'user_geo_ref_id': 2, 'zipcode': 12345, 'country': 'United States'}, 'Newton Falls': {'city': 'Newton Falls', 'state': 'Ohio', 'user_geo_ref_id': 4, 'zipcode': 44444, 'country': 'United States'}},
                           {1: {'first_name': 'Bob', 'last_name': 'Richards', 'user_id': 1, 'zipcode': 55555, 'birthday': datetime.date(2000, 1, 4), 'is_active': 0}, 2: {'first_name': 'Sarah', 'last_name': 'Jenkins', 'user_id': 2, 'zipcode': 12345, 'birthday': datetime.date(2000, 2, 2), 'is_active': 1}, 3: {'first_name': 'Frank', 'last_name': 'Williams', 'user_id': 3, 'zipcode': 56789, 'birthday': datetime.date(1972, 3, 3), 'is_active': 0}, 4: {'first_name': None, 'last_name': 'Thomas', 'user_id': 4, 'zipcode': 44444, 'birthday': datetime.date(1923, 1, 4), 'is_active': 1}}]

        self.assertItemsEqual(given_result, expected_result)

    def test_process_write_file_defined_column(self):
        data_set = [{'city': 'Young America', 'state': 'Minnesota', 'user_geo_ref_id': 1, 'zipcode': 55555, 'country': 'United States'}, {'city': 'Colton', 'state': 'California', 'user_geo_ref_id': 3, 'zipcode': 56789, 'country': 'United States'}, {'city': 'Schenectady', 'state': 'New York', 'user_geo_ref_id': 2, 'zipcode': 12345, 'country': 'United States'}, {'city': 'Newton Falls', 'state': 'Ohio', 'user_geo_ref_id': 4, 'zipcode': 44444, 'country': 'United States'}]
        YAMLParser().write_file(data_set, 'etlUnitTest', 'user_geo_ref', 'city')

        sample_file = os.path.join(self.data_dir, 'etlUnitTest/user_geo_ref_defined_column.yml')
        output_file = os.path.join(self.data_loc, 'etlUnitTest/user_geo_ref.yml')

        with open(sample_file) as f:
            expected_result = f.read()

        with open(output_file) as f:
            given_result = f.read()

        self.assertItemsEqual(given_result, expected_result)



__author__ = 'ameadows'

import unittest
import os

from etltest.utilities.settings_manager import SettingsManager
from etltest.code_generator import CodeGenerator


class CodeGeneratorTest(unittest.TestCase):

    def setUp(self):
        SettingsManager().first_run_test()
        self.out_dir = SettingsManager().find_setting('Locations', 'output')
        self.main_path = SettingsManager().get_file_location()
        self.test_dir = self.main_path + '/etltest/samples/test/'
        self.test_file = self.test_dir + '/dataMart/users_dim.yml'


    def test_generate_single_test_file(self):
        CodeGenerator(in_file=self.test_file).generate_test()
        # Testing the in_file option of code_generator
        sample_file = os.path.join(self.main_path, 'etltest/samples/output/DataMart/UsersDim.py')
        output_file = os.path.join(self.out_dir, 'DataMart/UsersDim.py')

        with open(output_file, 'r') as f:
            expected_result = f.read()

        with open(sample_file, 'r') as f:
            given_result = f.read()


        self.assertEqual(given_result, expected_result)

    def test_generate_multiple_test_file(self):
        # Testing the in_dir option of code_generator.
        CodeGenerator(in_dir=self.test_dir).generate_test()
        sample_dir = os.path.join(self.main_path, 'etltest/samples/output/DataMart/')
        output_dir = os.path.join(self.out_dir, 'DataMart')

        given_result = len([name for name in os.listdir(output_dir) if os.path.isfile(name)])
        expected_result = len([name for name in os.listdir(sample_dir) if os.path.isfile(name)])

        self.assertEqual(given_result, expected_result)

    # def test_generate_code_no_option(self):
    #     CodeGenerator(in_dir=self.test_dir).generate_test()
    #     # Testing if neither in_file or in_dir are provided.
    #     sample_dir = os.path.join(self.main_path, 'etltest/samples/output/DataMart/')
    #     output_dir = os.path.join(self.out_dir, 'DataMart')
    #
    #     given_result = len([name for name in os.listdir(output_dir) if os.path.isfile(name)])
    #     expected_result = len([name for name in os.listdir(sample_dir) if os.path.isfile(name)])
    #
    #     self.assertEqual(given_result, expected_result)


__author__ = 'ameadows'

"""
settingsManager handles building the standard directories used by etlTest,
copies the default settings files, and building the array that stores all the parameters
from the settings files.
"""
import logging
from ConfigParser import SafeConfigParser
from shutil import copyfile, copy2

import re
import os
import appdirs


class SettingsManager():

    def __init__(self):
        """
            This method initializes the log for SettingsManager as well as sets some static variables for file paths.
        """
        from settings import etltest_config, console

        self.log = logging.getLogger(name="SettingsManager")
        self.log.setLevel(etltest_config['logging_level'])
        self.log.addHandler(console)
        self.log.debug(u"Settings imported: {0:s}".format(etltest_config))
        self.app_name = etltest_config['app_name']
        self.app_author = etltest_config['app_author']
        self.data_dir = 'etltest/samples/data/'
        self.test_dir = 'etltest/samples/test/'
        self.settings_file = etltest_config['settings_file']
        self.connection_file = etltest_config['connection_file']
        self.tools_file = etltest_config['tools_file']


        self.variable_pattern = "\$\{.{0,10000}\}"

        self.user_settings = appdirs.user_data_dir(self.app_name, self.app_author)
        self.user_logging = appdirs.user_log_dir(self.app_name, self.app_author)

        self.path_loc = '/etltest/templates/settings/'

    def first_run_test(self):
        """
            Tests if the directories used for user settings exist or not.
              No parameters are passed from outside settingsManager.
        """

        if not os.path.isdir(self.user_settings):
            self.log.info('User settings directory does not exist.  Building now.')
            os.makedirs(self.user_settings)
            self.copy_settings_file(self.settings_file)
            self.copy_settings_file(self.connection_file)
            self.copy_settings_file(self.tools_file)

        else:
            self.log.info("User settings directory exists (%s)" % self.user_settings)

        self.make_dir(self.user_logging)

        data_location = self.find_setting('Locations', 'data')
        self.make_dir(data_location)

        test_location = self.find_setting('Locations', 'tests')
        self.make_dir(test_location)

        output_location = self.find_setting('Locations', 'output')
        self.make_dir(output_location)

        data_source = os.path.join(self.get_file_location(), self.data_dir)
        data_dest = data_location
        #Copying sample data files.
        self.copy_files(data_source, data_dest)

        test_source = os.path.join(self.get_file_location(), self.test_dir)
        test_dest = test_location
        #Copying sample test files.
        self.copy_files(test_source, test_dest)

    def copy_files(self, source, dest):
        """
        Takes files from the source and copies them to the destination directory.
        :param source: The source directory of the files.
        :type source: str
        :param dest:  Where the files need to be copied to.
        :type dest:  str
        """

        for root, dirs, files in os.walk(source):
            for file in files:
                self.log.debug("Trying to copy %s" % file)
                s = os.path.join(root, file)
                path_part = root.replace(source, "")
                dest_full = os.path.join(dest, path_part)
                d = os.path.join(dest_full, file)

                if not os.path.isdir(dest_full):
                    self.log.debug("Building destination directory: %s" % dest_full)
                    os.makedirs(dest_full)

                if not os.path.exists(d):
                    self.log.info(u"Copying file {0:s} to {1:s}".format(s, d))
                    copy2(s, d)

    def make_dir(self, dir):
        """
        Takes the given directory path and ensures that it exists.  If not, it will create it.
        :param dir: Directory path needing to be tested/created.
        :type dir: str
        """
        if not os.path.isdir(dir):
            self.log.info("Directory does not exist.  Building now.")
            os.makedirs(dir)
            self.log.debug("Directory is %s" % dir)
        else:
            self.log.debug("Directory exists (%s)" % dir)

    def get_settings(self):
        """
            Gets all the settings from the primary settings file (properties.cfg).
        """
        return self.read_settings_file(self.settings_file)

    def get_connections(self):
        """
            Gets all the connections from the primary connections file (connections.cfg).
        """

        return self.read_settings_file(self.connection_file)

    def get_tools(self):
        """
            Gets all the tools from the primary tools file (tools.yml)
        """

        from etltest.utilities.yaml_parser import YAMLParser

        return YAMLParser().read_file(os.path.join(self.user_settings, self.tools_file))

    def get_tool(self, tool_name):

        tools = self.get_tools()

        while True:
            tool = tools.next()

            if tool[tool_name]:
                return tool[tool_name]

    def find_setting(self, setting_section, setting_name):
        """
            Reads the config file and returns the given setting's value.
            :param setting_section: The properties file section name.
            :type setting_section: str
            :param setting_name: The property name
            :type setting_name: str
        """
        config = self.get_settings()

        try:
            config_var = config[setting_section][setting_name]
            return self.system_variable_replace(config_var)
        except Exception:
            return "Setting does not exist or something went wrong."

    @staticmethod
    def get_file_location():

        file_path = os.path.dirname(os.path.abspath(__file__))
        file_path = re.sub('/etltest/utilities', '', file_path)

        return file_path

    def read_settings_file(self, settings_file):
        """
            Parses file using the SafeConfigParser.  Will return the file as an array.
            :param settings_file: The file to be processed.
            :type settings_file:  str
            :returns: Array
        """
        parser = SafeConfigParser()
        parser.read(os.path.join(self.user_settings, settings_file))
        self.log.info(parser._sections)
        return parser._sections

    def system_variable_replace(self, parameter):
        """
            Takes the given parameter and checks to see if any system_variable in it exists on the system and then
            replaces the variable name with the value of the variable.
            :param parameter: The parameter needing to have a system variable replaced in (i.e. tool_path, code_path, etc.)
            :type parameter: str
            :return: Value of the parameter with all variables replaced.
        """
        variable_list = re.findall(self.variable_pattern, parameter)
        self.log.debug(u"Variable list:  {0:s}".format(variable_list))
        modified_param = str()
        for system_variable in variable_list:
            system_variable = re.sub('[\$\{\}]', '', system_variable)
            variable_value = str(os.environ.get(system_variable))

            self.log.debug(u"Found {0:s} and it has a value of {1:s}".format(system_variable, variable_value))

            if variable_value == "None":
                raise Exception(u"The system variable either does not exist or has a bad value. System variable: "
                                 u"{0:s}".format(system_variable))
            else:
                self.log.debug(u"Replacing ${0:s} with {1:s}".format(system_variable,
                               variable_value))
                modified_param = parameter.replace("${" + system_variable + "}", variable_value)
            self.log.debug(u"Final parameter value is: {0:s}".format(modified_param))
        if modified_param == '':
            return parameter
        else:
            return modified_param

    def copy_settings_file(self, filename):
        """
            Adds an alert to the log and then copies the file to the setting directory.
        :param filename:  The name of the file to be copied.
        :type filename: str
        """

        self.log.info(u"Copying default settings file to user directory. ({0:s}/{1:s})".format(self.user_settings,
                                                                                    filename))

        copyfile(self.get_file_location() + self.path_loc + filename,
                 self.user_settings + '/' + filename)


__author__ = 'coty, ameadows'

import logging
import yaml
import os

from etltest.utilities.settings_manager import SettingsManager

class YAMLParser():

    def __init__(self, in_file=None, in_dir=None):
        from settings import etltest_config, console
        """
            Initialization of method to setup the logging.
        """

        self.log = logging.getLogger(name="YAMLParser")
        self.log.setLevel(etltest_config['logging_level'])
        self.log.addHandler(console)

        self.in_file = in_file
        self.in_dir = in_dir
        self.data_dir = SettingsManager().find_setting('Locations', 'data')

        if in_file != None:
            self.read_file(self.in_file)

        if in_dir != None:
            self.read_dir(self.in_dir)

    def read_file(self, filename):
        """
        Reads a single file and turns it into an array for processing.
        :param filename The name of the file to be processed.
        :type filename str
        """

        with open(filename.strip(), 'r') as f:
            data = yaml.safe_load_all(f.read())
            self.log.debug("Reading file %s." % filename)
            return data

    def read_dir(self, dirname):
        """
        Reads in the directory and then processes it through all subdirectories until all files
        have been processed.
        :param dirname The name of the directory
        :type dirname str
        """

        yaml_results = []  # Full listing of all yaml file contents from the dirname.
        self.log.debug("Reading directory %s." % dirname)

        from os import walk
        for root, subFolders, files in walk(dirname):
            for file in files:
                filename = root + "/" + file
                self.log.debug("Processing file %s " % filename)
                yaml_results.extend(self.read_file(filename))
        return yaml_results

    def write_file(self, data_set, source, table, column=None):
        """
        This method takes the JSON data set and parses it into a YAML file into the data directory structure.
        :param data_set: The data to be processed into a YAML file.
        :type data_set: JSON Array
        :param source: The source where the data came from.
        :type source: str
        :param table: The table from the source where the data came from.
        :type table: str
        :param column: The column that the records should be identified by within tests.  Default is to number the records.
        :type column: str
        :return:  None
        """

        count = 1
        file_path = os.path.join(self.data_dir, source)
        filename = table + '.yml'

        self.log.info(u'Creating {0:s} if does not exist.'.format(file_path))

        if not os.path.isdir(file_path):
            os.makedirs(file_path, 0755)
            self.log.info(u'{0:s} created.'.format(file_path))

        os.chdir(file_path)
        self.log.info(u'Creating reference data file: {0:s}.'.format(filename))
        with open(filename, 'w') as f:
            # Need to take the dataset one record at a time and format accordingly.
            for record in data_set:
                # Need to check if the record is identified by a column value or if we just give it an identifier.
                if column is None:
                    f.write(str(count) + ':\n')
                else:
                    f.write(str(record[column]) + ':\n')

                for column_name, column_value in record.iteritems():
                    f.write('  ' + column_name + ': ' + str(column_value) + '\n')

                count += 1
        f.close()



__author__ = 'ameadows'

import unittest
import datetime

from etltest.data_connector import DataConnector
from etltest.utilities.settings_manager import SettingsManager


class DataConnectorTests(unittest.TestCase):

    def setUp(self):

        SettingsManager().first_run_test()

        self.source = 'etlUnitTest'
        self.table = 'users'
        self.records = [1, 2]

    def tearDown(self):
        DataConnector(self.source).truncate_data(self.table)

    def test_bad_connection(self):
        #Testing to see if a non existent connection will fail gracefully.
        records = [1, 2]

        with self.assertRaises(KeyError) as raises:
            DataConnector("BadConnection").generate_data(self.table, records)

    def test_generate_data_all(self):
        #Testing to see if the full data set is generated.
        given_result = DataConnector(self.source).generate_data(self.table)
        expected_result = [{'first_name': 'Bob', 'last_name': 'Richards', 'user_id': 1, 'zipcode': 55555,
                            'birthday': datetime.date(2000, 1, 4), 'is_active': 0}, {'first_name': 'Sarah',
                            'last_name': 'Jenkins', 'user_id': 2, 'zipcode': 12345,
                            'birthday': datetime.date(2000, 2, 2), 'is_active': 1},{'first_name': 'Frank',
                            'last_name': 'Williams', 'user_id': 3, 'is_active': 0, 'zipcode': 56789,
                            'birthday': datetime.date(1972, 3, 3)},{'first_name': None, 'last_name': 'Thomas',
                            'user_id': 4, 'is_active': 1, 'zipcode': 44444, 'birthday': datetime.date(1923, 1, 4)}]

        self.assertItemsEqual(given_result, expected_result)
    def test_generate_data_subset(self):
        #Testing to see if a subset of data is generated and not the full data set.
        given_result = DataConnector(self.source).generate_data(self.table, self.records)
        expected_result = [{'first_name': 'Bob', 'last_name': 'Richards', 'user_id': 1, 'zipcode': 55555,
                            'birthday': datetime.date(2000, 1, 4), 'is_active': 0}, {'first_name': 'Sarah',
                            'last_name': 'Jenkins', 'user_id': 2, 'zipcode': 12345,
                            'birthday': datetime.date(2000, 2, 2), 'is_active': 1}]

        self.assertEqual(given_result, expected_result)

    def test_generate_data_subset_non_existent_records(self):
        # Testing to see if a record doesn't exist, the data generator will only return valid records.
        records = [1, 2, 20]
        given_result = DataConnector(self.source).generate_data(self.table, records)
        expected_result = [{'first_name': 'Bob', 'last_name': 'Richards', 'user_id': 1, 'zipcode': 55555,
                            'birthday': datetime.date(2000, 1, 4), 'is_active': 0}, {'first_name': 'Sarah'
                            , 'last_name': 'Jenkins', 'user_id': 2, 'zipcode': 12345,
                            'birthday': datetime.date(2000, 2, 2), 'is_active': 1}]

        self.assertEqual(given_result, expected_result)

    def test_insert_data(self):
        # Testing to see if the data set will be inserted correctly.
        DataConnector(self.source).insert_data(self.table, self.records)
        given_result = DataConnector(self.source).select_data("all_columns", self.table, "user_id IN (1, 2)")
        expected_result = {'first_name': 'Bob', 'last_name': 'Richards', 'user_id': 1, 'zipcode': '55555'
                            , 'birthday': datetime.date(2000, 1, 4), 'is_active': 0}, {'first_name': 'Sarah', 'last_name': 'Jenkins'
                            , 'user_id': 2, 'zipcode': '12345', 'birthday': datetime.date(2000, 2, 2), 'is_active': 1}

        self.assertItemsEqual(given_result, expected_result)

    def test_truncate_data(self):
        # Testing to see if the data set will be removed completely.
        DataConnector(self.source).insert_data(self.table, self.records)
        DataConnector(self.source).truncate_data(self.table)
        given_result = DataConnector(self.source).select_data("all_columns", self.table)
        expected_result = []

        self.assertEqual(given_result, expected_result)

    def test_select_all_data(self):
        # Testing to see if the full data set will be selected correctly.
        records = [1, 2]
        DataConnector(self.source).insert_data(self.table, records)
        given_result = DataConnector(self.source).select_data("all_columns", self.table, "user_id = 2")
        expected_result = [{'first_name': 'Sarah', 'last_name': 'Jenkins', 'user_id': 2, 'zipcode': '12345', 'birthday': datetime.date(2000, 2, 2), 'is_active': 1}]

        self.assertEqual(given_result, expected_result)

    # def test_select_all_data_no_columns(self):
    #     # Testing that when all columns are selected, None value is passed.
    #     given_result = DataConnector(self.source).get_table(self.table, "all_columns")
    #     expected_result = DataConnector(self.source).get_table(self.table)
    #
    #     self.assertEquals(given_result, expected_result)

    def test_select_multiple_column_data(self):
        # Testing to see if the data set will be selected correctly.
        DataConnector(self.source).insert_data(self.table, self.records)
        given_result = DataConnector(self.source).select_data("first_name, last_name", self.table, "user_id = 1")
        expected_result = [{'first_name': 'Bob', 'last_name': 'Richards'}]

        self.assertEqual(given_result, expected_result)

    def test_select_single_column_data(self):
        # Testing to see if the data set will be selected correctly.
        DataConnector(self.source).insert_data(self.table, self.records)
        given_result = DataConnector(self.source).select_data("first_name", self.table, "user_id = 1")
        expected_result = [{'first_name': 'Bob'}]

        self.assertEqual(given_result, expected_result)

    def test_select_null_column_data(self):
        # Testing to see if the data set that has a null in it will be selected correctly.
        records = [1, 4]
        DataConnector(self.source).insert_data(self.table, records)
        given_result = DataConnector(self.source).select_data("first_name", self.table, "user_id IN (1, 4)")
        expected_result = [{'first_name': 'Bob'}, {'first_name': ''}]

        self.assertItemsEqual(given_result, expected_result)


s__author__ = 'coty, ameadows'
"""
    This file contains all the logic to connect to data sources using SQLAlchemy.
"""

import logging
import sys

from sqlalchemy.orm import sessionmaker
from sqlalchemy import create_engine, MetaData, Table, inspect

from etltest.utilities.settings import etltest_config, console
from etltest.utilities.yaml_parser import YAMLParser


class DataConnector():
    #TODO:  Need to determine if a data validation tool is required.  For instance, does record set match the table
    # structure?
    def __init__(self, conn_name):
        """
            This function initializes the data source connection so that data can be CRUDed.

            :param conn_name:  The name of the connection that the test suites reference.  This should match a
                                connection name from the connections settings file (default connections.cfg).
            :type conn_name: str
        """
        from etltest.utilities.settings_manager import SettingsManager

        self.log = logging.getLogger(name="DataConnector")
        self.log.setLevel(etltest_config['logging_level'])
        self.log.addHandler(console)

        self.conn_name = conn_name
        self.data_dir = SettingsManager().find_setting('Locations', 'data')

        connections = SettingsManager().get_connections()
        try:
            connection = connections[self.conn_name]
        except Exception:
            self.log.error("Connection %s does not exist, exiting." % conn_name)
            self.log.error(sys.exc_info()[0])
            raise

        self.engine = create_engine('{}://{}:{}@{}:{}/{}'.format(
            connection['type'],
            connection['username'],
            connection['password'],
            connection['hostname'],
            connection['port'],
            connection['dbname']
        ))

        self.meta = MetaData()
        self.insp = inspect(self.engine)

        session = sessionmaker(bind=self.engine)
        self.session = session()

        self.conn = self.engine.connect()

    def get_table(self, table_name, select_stmt=None):
        """
            This method gets a table from the data connection using SQLAlchemy's reflection capability. Columns in the
            select_stmt are included, with all other table columns excluded.
            :param table_name: The name of the table being reflected.
            :type table_name: str
            :param select_stmt: The fields being included in the query. Default None.
            :type select_stmt:  str
            :returns:  SQLAlchemy Table object
        """
        table = Table(table_name, self.meta)
        self.log.debug("Reflecting table %s" % table_name)

        if select_stmt == "all_columns":
            select_stmt = None

        self.insp.reflecttable(table, select_stmt)

        return table

    def insert_data(self, table_name, record_set=None):
        """
            This method inserts records into the given table for the specified connection.
            :param table_name: Name of the table we want to insert data into.
            :param table_name: str
            :param record_set: An array of record keys to retrieve the records to insert.
            :param record_set: arr
        """

        table = self.get_table(table_name)
        data = self.generate_data(table_name, record_set)
        insert = table.insert().values(data)
        self.log.debug("Inserting records for table %s" % table_name)

        self.conn.execute(insert)

    def truncate_data(self, table_name):
        #TODO: Need another method to only delete inserted records.  Only use this on single user environments.
        """
            This method deletes records from the given table by truncating the table.
            :param table_name:  Name of the table to be truncated.
            :param table_name:  str
        """
        table = self.get_table(table_name)
        delete = table.delete()
        self.log.debug("Truncating table %s." % table_name)

        self.conn.execute(delete)

    def select_data(self, select_stmt, from_stmt, where_stmt=None):
        #TODO:  How to handle complex WHERE clauses?
        #TODO:  Should we support sub-selects?
        #TODO:  Should we support joins?
        """
            This method queries records from the given table based on particular columns desired and the required
            filter criteria.  Currently only supports simple select queries.
            :param select_stmt:  Names of the columns to be returned from the query (SELECT clause).
            :param select_stmt: arr
            :param from_stmt:  Name of the table to be queried (FROM clause).
            :param from_stmt: str
            :param where_stmt:  The filter criteria for the query (WHERE clause)
            :param where_stmt:  arr
        """

        if select_stmt == "all_columns":
            table = self.get_table(from_stmt)
        else:
            table = self.get_table(from_stmt, select_stmt)

        if where_stmt is None:
            result = self.session.query(table)
        else:
            result = self.session.query(table).filter(where_stmt)

        self.log.debug(u"Executing query {0:s}".format(result))

        return self.to_json(result, table)

    def generate_data(self, table_name, records='all_records'):
        """
        :param table_name: Name of the table yaml file that will be retrieved.
        :param table_name: str
        :param records: The set of keys for the records that need to be retrieved and inserted into the connection.
        :param records: arr
        :return: Formatted data set for the the data connector method.
        """

        full_set = YAMLParser().read_file(self.data_dir + "/" + self.conn_name + "/" + table_name + ".yml")
        if records != 'all_records' and records is not None:
            subset = []
            for item in full_set:
                for record in records:
                    try:
                        subset.append(item[record])
                    except Exception:
                        self.log.info("Record %s is not in the full set." % record)
            return subset
        else:
            set = []
            for item in full_set:
                for key in item.keys():
                    set.append(item[key])
            return set

    def to_json(self, result_set, table):
        """
            This method converts the SQLAlchemy query result set into a JSON array.
            :param result_set: Results from a SQLAlchemy query.
            :param result_set: SQLAlchemy ResultSet Object.
            :param table:  Table object that returned the results.
            :param table:  SQLAlchemy Table Object.
            :returns:  Jsonified SQLAlchemy ResultSet.
        """
        results = [] # In the event nothing is in the set, return empty set.
        col_types = dict()
        for result in result_set:
            self.log.debug(u"Processing record: {0:s}".format(result))
            table_json = {}
            i = 0
            for column in table._columns:
                value = result[i]
                if value is None:
                    table_json[column.name] = str()
                else:
                    table_json[column.name] = value
                i += 1
            results.append(table_json)

        return results


#!/usr/bin/python
#
# This file was created by etlTest.
#

# These tests are also run as part of the following suites:
#
#    dataMart
#
# The following processes are executed for these tests:
#
#    PDI:
#      data_mart/user_dim_jb.kjb

import unittest
import datetime
from os import path

from etltest.data_connector import DataConnector
from etltest.process_executor import ProcessExecutor
from etltest.utilities.settings_manager import SettingsManager


class DataMartUsersDimTest(unittest.TestCase):

    def setUp(self):
          # Queries for loading test data.
            DataConnector("etlUnitTest").insert_data("users", [1, 2])

            PDI_settings = SettingsManager().get_tool("PDI")
            PDI_code_path = SettingsManager().system_variable_replace(PDI_settings["code_path"])
            ProcessExecutor("PDI").execute_process("job",
            path.join(PDI_code_path, "data_mart/user_dim_jb.kjb"))

    def tearDown(self):
       # Clean up testing environment.

        DataConnector("etlUnitTest").truncate_data("users")

    def testFirstNameNotLower(self):
        # Ensures that the first name field is not lower case after being processed.

        given_result = DataConnector("etlUnitTest").select_data("first_name",
                        "user_dim", "user_id = 2")

        expected_result = [{'first_name': 'sarah'}]

        self.assertNotEqual(given_result, expected_result)

    def testFirstNameUpper(self):
        # Test for process that upper cases the first name field of a users table record.

        given_result = DataConnector("etlUnitTest").select_data("first_name",
                        "user_dim", "user_id = 2")

        expected_result = [{'first_name': 'SARAH'}]

        self.assertEqual(given_result, expected_result)

    def testUserValidBirthday(self):
        # Test for valid birth dates.

        given_result = DataConnector("etlUnitTest").select_data("birthday",
                        "user_dim", "user_id IN (1, 2)")

        expected_result = [{'birthday': datetime.date(2000, 1, 4)}, {'birthday': datetime.date(2000, 2, 2)}]

        self.assertEqual(given_result, expected_result)

    def testIsActiveTrue(self):
        # Passes if is_active field is set to true.

        given_result = DataConnector("etlUnitTest").select_data("is_active",
                        "users", "user_id = 2")

        self.assertTrue(given_result)

    def testIsActiveFalse(self):
        # Passes if is_active field is set to false.

        given_result = DataConnector("etlUnitTest").select_data("is_active",
                        "users", "user_id = 1")

        self.assertFalse(given_result)


if __name__ == "__main__":
    unittest.main()


"""
    This file was created to house custom logging functions.
"""

__author__ = 'coty'

import logging


# Created a trace category for things less than debug to print on
TRACE = 5
logging.addLevelName(TRACE, 'TRACE')


def trace(self, message, *args, **kws):
    """
        This method creates the trace method on the logger that were using allowing me to use log.trace() in my code.
    """
    self.log(TRACE, message, *args, **kws)
logging.Logger.trace = trace

# This category was created as a result to the test option. When that opt is used, then this category prints the
# output from test logging.
TESTING = 21
logging.addLevelName(TESTING, 'TESTING')


def testing(self, message, *args, **kws):
    """
        This method creates the testing method on the logger that were using allowing me to use log.testing() in my
        code. This specific level is for the preview mode that the application can run it. It prints what code it
        generates instead of writing it to a file under the TESTING level.
    """
    self.log(TESTING, message, *args, **kws)
logging.Logger.testing = testing



__author__ = 'ameadows'

import unittest
import os
from shutil import copyfile

from etltest.process_executor import ProcessExecutor
from etltest.utilities.settings_manager import SettingsManager


class ProcessExecutorTests(unittest.TestCase):

    def setUp(self):
        SettingsManager().first_run_test()
        self.executor = ProcessExecutor('PDI')
        self.process_job = SettingsManager().system_variable_replace(
                                                                '${ETL_TEST_ROOT}/etltest/samples/etl/data_mart/user_dim_jb.kjb')
        self.process_trans = SettingsManager().system_variable_replace(
                                                                      '${ETL_TEST_ROOT}/etltest/samples/etl/data_mart/user_dim_load_tr.ktr')

        #self.process_job = "/opt/pentaho/pdi/src/com/redhat/bi/sandbox/test_failure_job_job.kjb"
        #self.process_trans = "/opt/pentaho/pdi/src/com/redhat/bi/sandbox/test_memory.ktr"

        shared_file = SettingsManager().system_variable_replace(
                                                               '${ETL_TEST_ROOT}/etltest/samples/etl/shared.xml')

        kettle_settings = SettingsManager().system_variable_replace('${ETL_TEST_ROOT}/.kettle')

        if os.path.isdir(kettle_settings) is False:
            os.mkdir(kettle_settings)

        shared_file_target = SettingsManager().system_variable_replace('${ETL_TEST_ROOT}/.kettle/shared.xml')
        if os.path.isfile(shared_file_target) is False:
            copyfile(shared_file, shared_file_target)

        self.mysql_driver = SettingsManager().system_variable_replace('${'
                                                                      'TOOL_PATH}/lib/mysql-connector-java-5.1.31-bin.jar')

    def test_sample_job_exists(self):
        given_result = os.path.isfile(self.process_job)
        expected_result = True

        self.assertEqual(given_result, expected_result)

    def test_sample_trans_exists(self):
        given_result = os.path.isfile(self.process_trans)
        expected_result = True

        self.assertEqual(given_result, expected_result)

    def test_mysql_driver_exists(self):
        given_result = os.path.isfile(self.mysql_driver)
        expected_result = True

        self.assertEqual(given_result, expected_result)

    def test_process_executor_job(self):
        given_result = self.executor.execute_process('job', self.process_job)
        expected_result = 0

        self.assertEqual(given_result, expected_result)

    def test_process_executor_trans(self):
        given_result = self.executor.execute_process('trans', self.process_trans)
        expected_result = 0

        self.assertEqual(given_result, expected_result)

    # def test_process_remote_execution_no_key(self):
    #
    # def test_process_remote_execution_key(self):


__author__ = 'coty'
"""
    This file contains all the logic necessary to execute the unit test code based on the parameters passed by the user.
"""

import logging

from utilities.settings import etltest_config, console


class CodeExecutor():

    def __init__(self, out_dir):
        """
            Here we initialize the CodeExecutor object, setting up the executor with custom logging and other
            parameters required.
        """

        self.log = logging.getLogger(name="CodeExecutor")
        self.log.setLevel(etltest_config['logging_level'])
        self.log.addHandler(console)

        self.out_dir = out_dir

    def execute(self, test_exec):
        """
        This method executes the generated tests.
        :param test_exec: Flag to determine if the tests really need to be executed or not.  If not, a list of tests
        that would be executed are shown.
        :type test_exec: boolean
        """

        from os import listdir
        from os.path import isfile, join

        # TODO: How to handle files that are not created tests? Should we list all files?

        files = [f for f in listdir(self.out_dir) if isfile(join(self.out_dir, f)) and f.endswith(".py")]

        self.log.debug(files)

        import sys
        import subprocess
        for f in files:
            file_path = "{0:s}/{1:s}".format(self.out_dir, f)

            if test_exec:
                self.log.info("Would execute {0:s}".format(file_path))
                return 0
            else:
                self.log.info(file_path)
                print subprocess.check_output([sys.executable, file_path])




__author__ = 'ameadows'

import unittest
from collections import OrderedDict
import inspect

import re
import os
import appdirs

from etltest.utilities.settings import etltest_config
from etltest.utilities.settings_manager import SettingsManager


class SettingsManagerTests(unittest.TestCase):

    def setUp(self):
        SettingsManager().first_run_test()
        self.app_name = etltest_config['app_name']
        self.app_author = etltest_config['app_author']
        self.data_dir = appdirs.user_data_dir(self.app_name, self.app_author)
        self.log_dir = appdirs.user_log_dir(self.app_name, self.app_author)
        self.settings_file = os.path.join(self.data_dir, etltest_config['settings_file'])
        self.connection_file = os.path.join(self.data_dir, etltest_config['connection_file'])
        self.data_location = SettingsManager().find_setting('Locations', 'data')
        self.tools_file = os.path.join(self.data_dir, etltest_config['tools_file'])
        self.copy_file = os.path.join(self.data_dir, 'copy.test')
        self.tests_location = SettingsManager().find_setting('Locations', 'tests')
        self.output_location = SettingsManager().find_setting('Locations', 'output')

        self.maxDiff = None

    def test_data_dir_exists(self):
        assert os.path.exists(self.data_dir)

    def test_log_dir_exists(self):
        assert os.path.exists(self.log_dir)

    def test_settings_file_exists(self):
        assert os.path.isfile(self.settings_file)

    def test_tools_file_exists(self):
        assert os.path.isfile(self.tools_file)

    def test_data_samples_exists(self):
        assert os.path.isdir(self.data_location)

    def test_tests_dir_exists(self):
        assert os.path.isdir(self.tests_location)

    def test_output_dir_exists(self):
        assert os.path.isdir(self.output_location)

    def test_get_config_settings(self):
        given_result = SettingsManager().get_settings()
        expected_result = OrderedDict([('Locations', OrderedDict([('__name__', 'Locations'),
                                       ('tests', '${ETL_TEST_ROOT}/Documents/etlTest/tests'), ('data'
                                       , '${ETL_TEST_ROOT}/Documents/etlTest/data'), ('output',
                                       '${ETL_TEST_ROOT}/Documents/etlTest/output')])), ('Results',
                                       OrderedDict([('__name__', 'Results'), ('verbose',
                                       'True'), ('failurerate', '10'), ('reporttype', 'Normal')]))])

        self.assertEqual(given_result, expected_result)

    def test_get_connections(self):
        given_result = SettingsManager().get_connections()
        expected_result = OrderedDict([('etlUnitTest', OrderedDict([('__name__', 'etlUnitTest'),
                                                        ('hostname', '127.0.0.1'), ('username', 'root')
                                                        , ('password', ''), ('port', '3306'), ('type', 'mysql')
                                                        , ('dbname', 'etlUnitTest')]))])

        self.assertEqual(given_result, expected_result)

    def test_get_tools(self):
        given_result = list(SettingsManager().get_tools())
        expected_result = [{'PDI': {'code_path': '${ETL_TEST_ROOT}/etltest/samples/etl/', 'script_types': [{'script':
                                                                                                                'kitchen.sh', 'type': 'job'}, {'script': 'pan.sh', 'type': 'trans'}], 'port': None, 'password': None, 'private_key': '~/.ssh/id_rsa', 'process_param': '/file:', 'user_name': None, 'host_name': 'localhost', 'logging_filename_format': '${name}_%Y-%m-%d', 'tool_path': '${TOOL_PATH}', 'params': '/level: Detailed'}}]

        self.assertItemsEqual(given_result, expected_result)

    def test_get_tool(self):
        given_result = list(SettingsManager().get_tool('PDI'))
        expected_result = ['host_name', 'user_name', 'password', 'port', 'private_key', 'tool_path', 'script_types',
                           'params', 'process_param', 'code_path', 'logging_filename_format']
        self.assertItemsEqual(given_result, expected_result)

    def test_find_single_setting(self):
        given_result = SettingsManager().find_setting('Locations', 'tests')
        expected_result = os.environ.get('ETL_TEST_ROOT') + "/Documents/etlTest/tests"

        self.assertEqual(given_result, expected_result)

    def test_fail_find_single_setting(self):
        given_result = SettingsManager().find_setting('Location', 'tests')
        expected_result = 'Setting does not exist or something went wrong.'

        self.assertEqual(given_result, expected_result)

    def test_get_file_location(self):
        given_result = SettingsManager().get_file_location()
        expected_result = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
        expected_result = re.sub('/etltest/test/utilities', '', expected_result)

        self.assertEqual(given_result, expected_result)

    def test_system_variable_replace(self):
        parameter = "${ETL_TEST_ROOT}/this_is_a_test/file.txt"
        given_result = SettingsManager().system_variable_replace(parameter)
        expected_result = str(os.environ.get('ETL_TEST_ROOT')) + "/this_is_a_test/file.txt"

        self.assertEqual(given_result, expected_result)

    def test_bad_system_variable_replace(self):
        parameter = "${ETL_TEST_TOOT}/this_is_a_test/file.txt"
        expected_result = "The system variable either does not exist or has a bad value. System variable: ETL_TEST_TOOT"

        with self.assertRaises(Exception) as raises:
            SettingsManager().system_variable_replace(parameter)

        self.assertEqual(raises.exception.message, expected_result)

    def test_copy_settings_file(self):
        SettingsManager().copy_settings_file('copy.test')
        given_result = os.path.isfile(self.copy_file)
        expected_result = True

        self.assertEqual(given_result, expected_result)

    def test_no_system_variable_replace(self):
        parameter = "no_parameter/this_is_a_test/file.txt"
        given_result = SettingsManager().system_variable_replace(parameter)
        expected_result = "no_parameter/this_is_a_test/file.txt"

        self.assertEqual(given_result, expected_result)

    def test_copy_files(self):

        given_result = os.path.isfile(os.path.join(self.data_location, "etlUnitTest/users.yml"))
        expected_result = True

        self.assertEqual(given_result, expected_result)



__author__ = 'ameadows, coty'

import sys

from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand

import etltest


class Tox(TestCommand):
    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        #import here, cause outside the eggs aren't loaded
        import tox
        errcode = tox.cmdline(self.test_args)
        sys.exit(errcode)

setup(
    name='etlTest',
    version=etltest.__version__,
    url='https://github.com/OpenDataAlex/etlTest',
    license='GNU GENERAL PUBLIC LICENSE Version 3',
    description='Automated and tool agnostic data integration testing tool.',
    author='Alex Meadows, Coty Sutherland',
    author_email='alexmeadows@bluefiredatasolutions.com',
    packages=find_packages(),
    install_requires=[
        'appdirs==1.3.0',
        'docutils==0.11',
        'ecdsa==0.11',
        'Jinja2==2.7.2',
        'MarkupSafe==0.19',
        'MySQL-python==1.2.5',
        'paramiko==1.14.0',
        'Pygments==1.6',
        'py==1.4.20',
        'PyYAML==3.11',
        'Sphinx==1.2.2',
        'SQLAlchemy==0.9.3',
        'subprocess32==3.2.6',
        'tox==1.7.1',
        'unittest-xml==0.2.2',
        'unittest-xml-reporting==1.9.0',
        'virtualenv==1.11.5',
        'wsgiref==0.1.2',
        'xmlunittest==0.2.0'
    ],
    classifiers=[
        'Programming Language :: Python',
        'Development Status :: 4 - Beta',
        'Natural Language :: English',
        'Environment :: Console',
        'Intended Audience :: Developers'
    ],
    cmdclass={'tox': Tox},
    keywords='etl data_integration testing automation',
    test_suite='etltest.test',
    entry_points={
        'console_scripts': [
            'etlTest = etltest.etlTest:main',
            'etltest = etltest.etlTest:main',
        ]
    },
    package_data={
        'etlTest': ['.etltest-settings.yml'],
    },
    data_files=[
        ('', [
                                            '.etltest-settings.yml',
                                            'CONTRIBUTING.md',
                                            'LICENSE',
                                            'README.md'
        ]),
        ('etltest/samples/data/etlUnitTest', ['etltest/samples/data/etlUnitTest/users.yml']),
        ('etltest/samples/etl', ['etltest/samples/etl/shared.xml']),
        ('etltest/samples/etl/data_mart', [
                                            'etltest/samples/etl/data_mart/user_dim_jb.kjb',
                                            'etltest/samples/etl/data_mart/user_dim_load_tr.ktr'
        ]),
        ('etltest/samples/output/DataMart', ['etltest/samples/output/DataMart/UsersDim.py']),
        ('etltest/samples/output/main', [
                                            'etltest/samples/output/main/help_arg.txt',
                                            'etltest/samples/output/main/in_dir_generation.txt',
                                            'etltest/samples/output/main/in_file_generation.txt',
                                            'etltest/samples/output/main/no_args.txt'
        ]),
        ('etltest/samples/test/dataMart', ['etltest/samples/test/dataMart/users_dim.yml']),
        ('etltest/templates/output', [
                                        'etltest/templates/output/fixture.jinja2',
                                        'etltest/templates/output/suite.jinja2',
                                        'etltest/templates/output/test.jinja2'
        ]),
        ('etltest/templates/settings', [
                                        'etltest/templates/settings/connections.cfg',
                                        'etltest/templates/settings/copy.test',
                                        'etltest/templates/settings/properties.cfg',
                                        'etltest/templates/settings/tools.yml'
        ]),
    ],
)



#!/usr/bin/python
__author__ = 'coty, ameadows'

import sys
import argparse

from utilities.settings_manager import SettingsManager
from data_connector import DataConnector
from utilities.yaml_parser import YAMLParser


def create_parser():
    """
    Generates a command line parser with args.
    :return: parser
    """
    parser = argparse.ArgumentParser(
        description='Automated data integration test generator and executor.',
        epilog='etlTest -d /your/test_file/directory/ -g'
    )

    parser.add_argument('-f', '--infile', nargs='?', type=str, dest='in_file', help='Specify the input file')
    parser.add_argument('-d', '--indir', nargs='?', type=str, dest='in_dir', help='Specify the input directory')
    parser.add_argument('-o', '--outdir', nargs='?', type=str, dest='out_dir', help='Specify the output directory')
    parser.add_argument('-g', '--gen', dest='gen_code', default=False, action='store_true',
                        help='Generate new test code')
    parser.add_argument('-e', '--exec', dest='exec_code', default=False, action='store_true',
                        help='Execute test code')
    parser.add_argument('-t', '--test', dest='test_run', default=False, action='store_true',
                        help='Run app as tests.  Does not persist the generated or executed code.')
    parser.add_argument('-v', '--version', action='version', version='%(prog)s 1.1')

    # subparsers = parser.add_subparsers(title="Reference Data Handler",
    #                                    help="Import and synchronize reference data sets",
    #                                    dest="reference_handler")
    # subparsers.required = False

    return parser


def create_parser_ref():

    # Create parser for handling reference data for tests.
    parser_ref = argparse.ArgumentParser(
        description='Reference data handler for etlTest.',
        epilog='etlTest -s yourSource -t yourSourceTable -c yourTableIdentifyingColumn -i'
    )
    parser_ref.add_argument('-s', '--source', nargs='?', type=str, dest='source',
                            help='Name of the source from the connections.cfg file')
    parser_ref.add_argument('-t', '--table', nargs='?', type=str, dest='table',
                            help='Name of the table from the named source')
    parser_ref.add_argument('-c', '--column', nargs='?', type=str, dest='column',
                            help='Name of the key column from the named table.  Used to identify records.')
    parser_ref.add_argument('-i', '--import', dest='import_data', default=False, action='store_true',
                            help='Import data from the named source table.')
    parser_ref.add_argument('-r', '--refresh', nargs='?', type=str, choices=['ours', 'theirs'], dest='refresh_data',
                            help='Refreshes the stored reference data.  If ours, we will keep our data and refresh the '
                                 'source.  If theirs, we will drop our data and refresh from the source.')

    return parser_ref


def main():

    """
        This class is the entry point for the application. It takes the arguments, validates them, and passes
        them on to the appropriate classes to continue execution.

        Here are the main functions of this application.
        1) Take in YAML
        2) Generate code from that YAML
        3) Execute that code so that we can take advantage of the unittest libraries
        4) Handle reference data from sources/targets that needs to be kept in sync
    """
    parser = create_parser()
    parser_ref = create_parser_ref()
    args, extra = parser.parse_known_args()
    ref_args, ref_extra = parser_ref.parse_known_args()

    # no arguments, print usage
    if len(sys.argv) < 2:
        parser.print_help()
        parser_ref.print_help()

    if 'ref' in extra:
        # We require a source connection to work with.
        if ref_args.source is None:
            parser_ref.error("A source is required to work with reference data.  Please provide an existing data source.")

        # We also need a table name to pull data from.
        if ref_args.table is None:
            parser_ref.error("A source table is required to work with reference data.  "
                         "Please provide an existing reference table.")

        # It's okay if a column is not specified for identifying the records.  We'll number them.

        # The primary function of the reference handler is to import data from a source and store it as a YAML file.
        if ref_args.import_data or ref_args.refresh_data == 'theirs':
            # First we need to build our data set.  Pulling the data.
            print(u"Reading data from {0:s}.{1:s}".format(ref_args.source, ref_args.table))

            data_set = DataConnector(ref_args.source).select_data("all_columns", ref_args.table)

            # Now we need to process the data and turn it into a YAML file.

            print(u"Generating YAML file {0:s}/{1:s}.yml using {2:s} as the record identifier.".format(ref_args.source, ref_args.table, ref_args.column))
            YAMLParser().write_file(data_set, ref_args.source, ref_args.table, ref_args.column)

        if ref_args.refresh_data == 'ours':
            # We need to take our reference data file and try to load it into the data target.

            # First we need to truncate the table.
            print(u"Truncate table {0:s}.{1:s}".format(ref_args.source, ref_args.table))
            DataConnector(ref_args.source).truncate_data(ref_args.table)

            # And now we can load all our data from the reference data file.
            print(u"Loading data from reference data file.")
            DataConnector(ref_args.source).insert_data(ref_args.table)

    else:
        # validating args
        if args.in_file and args.in_dir:
            parser.error("Options infile and indir are mutually exclusive. Please choose one.")

        # Has a custom output directory been set?  If not, use default.
        if args.out_dir:
            out_dir = args.out_dir
        else:
            out_dir = SettingsManager().find_setting('Locations', 'output')

        if args.gen_code:
            from code_generator import CodeGenerator

            if args.in_file:
                print(u"Attempting to process: {0:s}".format(args.in_file))
                CodeGenerator(in_file=args.in_file, out_dir=out_dir).generate_test()

            if args.in_dir:
                print(u"Attempting to process: {0:s}".format(args.in_dir))
                CodeGenerator(in_dir=args.in_dir, out_dir=out_dir).generate_test()

            # TODO: Decide if there should be a way to check if generated code should be updated or not.
            # TODO: Fully enable test run capability.

        if args.exec_code:
            from code_executor import CodeExecutor
            e = CodeExecutor(out_dir)
            e.execute(args.test_run)

if __name__ == "__main__":
    SettingsManager().first_run_test()
    main()


"""
    This file reads the etltest-settings.yml file and loads application settings.

    Having a settings file in the users home directory overrides the one in the application directory.
"""

__author__ = 'coty'

import logging
import pprint

import os
import yaml

from settings_manager import SettingsManager



# Begin CustomLogging
# this import adds the additional log levels I created

# create console handler and set level to info
console = logging.StreamHandler()
console.setLevel(logging.DEBUG)

# create formatter
formatter = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")

# set formatter
console.setFormatter(formatter)
# End CustomLogging

etltest_config = {}

settings_log = logging.getLogger(name="settings")
settings_log.setLevel(logging.DEBUG)
settings_log.addHandler(console)

pp = pprint.PrettyPrinter(indent=4)

settings_filename = os.path.join(SettingsManager.get_file_location(), '.etltest-settings.yml')

settings_fs_locs = ["{}/{}".format('', settings_filename), "".join(settings_filename)]

settings_loaded = False
for the_path in settings_fs_locs:
    settings_log.debug("Attempting to load {}".format(the_path))
    try:
        with open(the_path, 'r') as f:
            prop_list = yaml.load(f.read())
            for key, value in prop_list.items():
                etltest_config[key] = value

            settings_log.debug(u"Settings loaded from {0:s}, {1:s}".format(the_path, etltest_config))
            settings_loaded = True
            break
    except (OSError, IOError) as e:
        settings_log.warn("{} {}".format(e.strerror, the_path))


if not settings_loaded:
    settings_log.warn("Could not find settings file in {}. Using defaults where present.".format(','.join(settings_fs_locs)))
    # For now there is nothing in the settings file that is required for operation. Removing the exit call.
    # exit()

# INFO = 20
# TESTING = 21
# DEBUG = 10
# TRACE = 5
try:
    console.level = etltest_config['logging_level']
except KeyError:
    etltest_config['logging_level'] = 20  # set default level to info
    console.level = etltest_config['logging_level']
